@@ -1,6 +1,6 @@
 #Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
-#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/graphics/widgetbase.py
+#history https://hg.reportlab.com/hg-public/reportlab/log/tip/src/reportlab/graphics/widgetbase.py
 __version__='3.3.0'
 __doc__='''Base class for user-defined graphical widgets'''
 
@@ -207,9 +207,9 @@ class TypedPropertyCollection(PropHolder):
         line[(3,x)] is backed by line[(3,)] == line[3] & line
     """
 
-    def __init__(self, exampleClass):
+    def __init__(self, exampleClass, **kwds):
         #give it same validation rules as what it holds
-        self.__dict__['_value'] = exampleClass()
+        self.__dict__['_value'] = exampleClass(**kwds)
         self.__dict__['_children'] = {}
 
     def wKlassFactory(self,Klass):
@@ -218,7 +218,7 @@ class TypedPropertyCollection(PropHolder):
                 try:
                     return self.__class__.__bases__[0].__getattr__(self,name)
                 except:
-                    parent = self.__propholder_parent__()
+                    parent = self.parent
                     c = parent._children
                     x = self.__propholder_index__
                     while x:
@@ -226,6 +226,9 @@ class TypedPropertyCollection(PropHolder):
                             return getattr(c[x],name)
                         x = x[:-1]
                     return getattr(parent,name)
+            @property
+            def parent(self):
+                return self.__propholder_parent__()
         return WKlass
 
     def __getitem__(self, x):
@@ -262,8 +265,8 @@ class TypedPropertyCollection(PropHolder):
         return key in self._children
 
     def __setitem__(self, key, value):
-        msg = "This collection can only hold objects of type %s" % self._value.__class__.__name__
-        assert isinstance(value, self._value.__class__), msg
+        assert isinstance(value, self._value.__class__), (
+            "This collection can only hold objects of type %s" % self._value.__class__.__name__)
 
     def __len__(self):
         return len(list(self._children.keys()))
@@ -286,8 +289,8 @@ class TypedPropertyCollection(PropHolder):
 
     def setVector(self,**kw):
         for name, value in kw.items():
-            for i in range(len(value)):
-                setattr(self[i],name,value[i])
+            for i, v in enumerate(value):
+                setattr(self[i],name,v)
 
     def __getattr__(self,name):
         return getattr(self._value,name)
@@ -295,6 +298,16 @@ class TypedPropertyCollection(PropHolder):
     def __setattr__(self,name,value):
         return setattr(self._value,name,value)
 
+def tpcGetItem(obj,x):
+    '''return obj if it's not a TypedPropertyCollection else obj[x]'''
+    return obj[x] if isinstance(obj,TypedPropertyCollection) else obj
+
+def isWKlass(obj):
+    if not hasattr(obj,'__propholder_parent__'): return
+    ph = obj.__propholder_parent__
+    if not isinstance(ph,weakref_ref): return
+    return isinstance(ph(),TypedPropertyCollection)
+
 ## No longer needed!
 class StyleProperties(PropHolder):
     """A container class for attributes used in charts and legends.
@@ -496,6 +509,107 @@ class Sizer(Widget):
             g.add(elem)
         return g
 
+class CandleStickProperties(PropHolder):
+    _attrMap = AttrMap(
+        strokeWidth = AttrMapValue(isNumber, desc='Width of a line.'),
+        strokeColor = AttrMapValue(isColorOrNone, desc='Color of a line or border.'),
+        strokeDashArray = AttrMapValue(isListOfNumbersOrNone, desc='Dash array of a line.'),
+        crossWidth = AttrMapValue(isNumberOrNone,desc="cross line width",advancedUsage=1),
+        crossLo = AttrMapValue(isNumberOrNone,desc="cross line low value",advancedUsage=1),
+        crossHi = AttrMapValue(isNumberOrNone,desc="cross line high value",advancedUsage=1),
+        boxWidth = AttrMapValue(isNumberOrNone,desc="width of the box part",advancedUsage=1),
+        boxFillColor = AttrMapValue(isColorOrNone, desc='fill color of box'),
+        boxLo = AttrMapValue(isNumberOrNone,desc="low value of the box",advancedUsage=1),
+        boxMid = AttrMapValue(isNumberOrNone,desc="middle box line value",advancedUsage=1),
+        boxHi = AttrMapValue(isNumberOrNone,desc="high value of the box",advancedUsage=1),
+        position = AttrMapValue(isNumberOrNone,desc="position of the candle",advancedUsage=1),
+        chart = AttrMapValue(None,desc="our chart",advancedUsage=1),
+        candleKind = AttrMapValue(OneOf('vertical','horizontal'),desc="candle direction",advancedUsage=1),
+        axes = AttrMapValue(SequenceOf(isString,emptyOK=0,lo=2,hi=2),desc="candle direction",advancedUsage=1),
+        )
+
+    def __init__(self,**kwds):
+        self.strokeWidth = kwds.pop('strokeWidth',1)
+        self.strokeColor = kwds.pop('strokeColor',colors.black)
+        self.strokeDashArray = kwds.pop('strokeDashArray',None)
+        self.crossWidth = kwds.pop('crossWidth',5)
+        self.crossLo = kwds.pop('crossLo',None)
+        self.crossHi = kwds.pop('crossHi',None)
+        self.boxWidth = kwds.pop('boxWidth',None)
+        self.boxFillColor = kwds.pop('boxFillColor',None)
+        self.boxLo = kwds.pop('boxLo',None)
+        self.boxMid = kwds.pop('boxMid',None)
+        self.boxHi = kwds.pop('boxHi',None)
+        self.position = kwds.pop('position',None)
+        self.candleKind = kwds.pop('candleKind','vertical')
+        self.axes = kwds.pop('axes',['categoryAxis','valueAxis'])
+        chart = kwds.pop('chart',None)
+        self.chart = weakref_ref(chart) if chart else (lambda:None)
+
+    def __call__(self,_x,_y,_size,_color):
+        '''the symbol interface'''
+        chart = self.chart()
+        xA = getattr(chart,self.axes[0])
+        _xScale = getattr(xA,'midScale',None)
+        if not _xScale: _xScale = getattr(xA,'scale')
+        xScale = lambda x: _xScale(x) if x is not None else None
+        yA = getattr(chart,self.axes[1])
+        _yScale = getattr(yA,'midScale',None)
+        if not _yScale: _yScale = getattr(yA,'scale')
+        yScale = lambda x: _yScale(x) if x is not None else None
+        G = shapes.Group().add
+        strokeWidth = self.strokeWidth
+        strokeColor = self.strokeColor
+        strokeDashArray = self.strokeDashArray
+        crossWidth = self.crossWidth
+        crossLo = yScale(self.crossLo)
+        crossHi = yScale(self.crossHi)
+        boxWidth = self.boxWidth
+        boxFillColor = self.boxFillColor
+        boxLo = yScale(self.boxLo)
+        boxMid = yScale(self.boxMid)
+        boxHi = yScale(self.boxHi)
+        position = xScale(self.position)
+        candleKind = self.candleKind
+        haveBox = None not in (boxWidth,boxLo,boxHi)
+        haveLine = None not in (crossLo,crossHi)
+        def aLine(x0,y0,x1,y1):
+            if candleKind!='vertical':
+                x0,y0 = y0,x0
+                x1,y1 = y1,x1
+            G(shapes.Line(x0,y0,x1,y1,strokeWidth=strokeWidth,strokeColor=strokeColor,strokeDashArray=strokeDashArray))
+        if haveBox:
+            boxLo, boxHi = min(boxLo,boxHi), max(boxLo,boxHi)
+        if haveLine:
+            crossLo, crossHi = min(crossLo,crossHi), max(crossLo,crossHi)
+            if not haveBox or crossLo>=boxHi or crossHi<=boxLo:
+                aLine(position,crossLo,position,crossHi)
+                if crossWidth is not None:
+                    aLine(position-crossWidth*0.5,crossLo,position+crossWidth*0.5,crossLo)
+                    aLine(position-crossWidth*0.5,crossHi,position+crossWidth*0.5,crossHi)
+            elif haveBox:
+                if crossLo<boxLo:
+                    aLine(position,crossLo,position,boxLo)
+                    aLine(position-crossWidth*0.5,crossLo,position+crossWidth*0.5,crossLo)
+                if crossHi>boxHi:
+                    aLine(position,boxHi,position,crossHi)
+                    aLine(position-crossWidth*0.5,crossHi,position+crossWidth*0.5,crossHi)
+        if haveBox:
+            x = position - boxWidth*0.5
+            y = boxLo
+            h = boxHi - boxLo
+            w = boxWidth
+            if candleKind!='vertical':
+                x, y, w, h = y, x, h, w
+            G(shapes.Rect(x,y,w,h,strokeColor=strokeColor,strokeWidth=strokeWidth,strokeDashArray=strokeDashArray,fillColor=boxFillColor))
+
+            if boxMid is not None:
+                aLine(position-0.5*boxWidth,boxMid,position+0.5*boxWidth,boxMid)
+        return G.__self__
+
+def CandleSticks(**kwds):
+    return TypedPropertyCollection(CandleStickProperties,**kwds)
+
 def test():
     from reportlab.graphics.charts.piecharts import WedgeProperties
     wedges = TypedPropertyCollection(WedgeProperties)
