@@ -1,6 +1,6 @@
 #Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
-#history https://bitbucket.org/rptlab/reportlab/history-node/tip/src/reportlab/platypus/flowables.py
+#history https://hg.reportlab.com/hg-public/reportlab/log/tip/src/reportlab/platypus/flowables.py
 __version__='3.3.0'
 __doc__="""
 A flowable is a "floating element" in a document whose exact position is determined by the
@@ -30,19 +30,20 @@ from reportlab.lib.colors import red, gray, lightgrey
 from reportlab.lib.rl_accel import fp_str
 from reportlab.lib.enums import TA_LEFT, TA_CENTER, TA_RIGHT, TA_JUSTIFY
 from reportlab.lib.styles import _baseFontName
-from reportlab.lib.utils import strTypes
+from reportlab.lib.utils import strTypes, safer_globals
+from reportlab.lib.abag import ABag
 from reportlab.pdfbase import pdfutils
 from reportlab.pdfbase.pdfmetrics import stringWidth
 from reportlab.rl_config import _FUZZ, overlapAttachedSpace, ignoreContainerActions, listWrapOnFakeWidth
 from reportlab import xrange
 import collections
 
-__all__ = '''TraceInfo Flowable XBox Preformatted Image NullDraw Spacer UseUpSpace PageBreak SlowPageBreak
-            PageBreakIfNotEmpty CondPageBreak KeepTogether Macro CallerMacro ParagraphAndImage FailOnWrap
-            FailOnDraw HRFlowable PTOContainer KeepInFrame ImageAndFlowables AnchorFlowable FrameBG
-            FrameSplitter BulletDrawer DDIndenter LIIndenter ListItem ListFlowable TopPadder DocAssign
-            DocExec DocPara DocAssert DocIf DocWhile SetTopFlowables splitLines splitLine SetPageTopFlowables
-            BalancedColumns'''.split()
+__all__ = '''AnchorFlowable BalancedColumns BulletDrawer CallerMacro CondPageBreak DDIndenter DocAssert
+        DocAssign DocExec DocIf DocPara DocWhile FailOnDraw FailOnWrap Flowable FrameBG FrameSplitter
+        HRFlowable Image ImageAndFlowables KeepInFrame KeepTogether LIIndenter ListFlowable ListItem
+        Macro NullDraw PTOContainer PageBreak PageBreakIfNotEmpty ParagraphAndImage Preformatted
+        SetPageTopFlowables SetTopFlowables SlowPageBreak Spacer TopPadder TraceInfo UseUpSpace XBox
+        splitLine splitLines'''.split()
 
 class TraceInfo:
     "Holder for info about where an object originated"
@@ -190,8 +191,12 @@ class Flowable:
             r = r[:maxLen]
         return "<%s at %s%s>%s" % (self.__class__.__name__, hex(id(self)), self._frameName(), r)
 
+    @property
+    def _doctemplate(self):
+        return getattr(getattr(self,'canv',None),'_doctemplate',None)
+
     def _doctemplateAttr(self,a):
-        return getattr(getattr(getattr(self,'canv',None),'_doctemplate',None),a,None)
+        return getattr(self._doctemplate,a,None)
 
     def _frameName(self):
         f = getattr(self,'_frame',None)
@@ -652,18 +657,14 @@ def _flowableSublist(V):
     return V
 
 class _ContainerSpace:  #Abstract some common container like behaviour
-    def getSpaceBefore(self):
-        for c in self._content:
+    def getSpaceBefore(self,content=None):
+        for c in (self._content if content is None else content):
             if not hasattr(c,'frameAction'):
                 return c.getSpaceBefore()
         return 0
 
     def getSpaceAfter(self,content=None):
-        #this needs 2.4
-        #for c in reversed(content or self._content):
-        reverseContent = (content or self._content)[:]
-        reverseContent.reverse()
-        for c in reverseContent:
+        for c in reversed(self._content if content is None else content):
             if not hasattr(c,'frameAction'):
                 return c.getSpaceAfter()
         return 0
@@ -707,22 +708,25 @@ class KeepTogether(_ContainerSpace,Flowable):
     def split(self, aW, aH):
         if getattr(self,'_wrapInfo',None)!=(aW,aH): self.wrap(aW,aH)
         S = self._content[:]
-        cf = getattr(self,'_frame',None)
-        if cf: atTop = getattr(cf,'_atTop',None)
+        cf = atTop = getattr(self,'_frame',None)
+        if cf:
+            atTop = getattr(cf,'_atTop',None)
+            cAW = cf._width
+            cAH = cf._height
         C0 = self._H>aH and (not self._maxHeight or aH>self._maxHeight)
         C1 = (self._H0>aH) or C0 and atTop
         if C0 or C1:
             fb = False
+            panf = self._doctemplateAttr('_peekNextFrame')
+            if cf and panf:
+                nf = panf()
+                nAW = nf._width
+                nAH = nf._height
             if C0 and not (self.splitAtTop and atTop):
+                fb = not (atTop and cf and nf and cAW>=nAW and cAH>=nAH)
+            elif nf and nAW>=cf._width and nAH>=self._H:
                 fb = True
-            else:
-                panf = self._doctemplateAttr('_peekNextFrame')
-                if cf and panf:
-                    nf = panf()
-                    nAW = nf._width
-                    nAH = nf._height
-                    if nAW>=cf._width and nAH>=self._H:
-                        fb = True
+
             S.insert(0,(self.FrameBreak if fb else self.NullActionFlowable)())
         return S
 
@@ -751,7 +755,7 @@ class Macro(Flowable):
     def wrap(self, availWidth, availHeight):
         return (0,0)
     def draw(self):
-        exec(self.command, globals(), {'canvas':self.canv})
+        exec(self.command, safer_globals(), {'canvas':self.canv})
 
 def _nullCallable(*args,**kwds):
     pass
@@ -920,7 +924,7 @@ class _Container(_ContainerSpace):  #Abstract some common container like behavio
         frame = getattr(self,'_frame',None)
         for c in content:
             if not ignoreContainerActions and isinstance(c,ActionFlowable):
-                c.apply(self.canv._doctemplate)
+                c.apply(canv._doctemplate)
                 continue
             if isinstance(c,Indenter):
                 x += c.left*scale
@@ -933,22 +937,22 @@ class _Container(_ContainerSpace):  #Abstract some common container like behavio
                 if not getattr(c,'_SPACETRANSFER',False):
                     h += max(s-pS,0)
             y -= h
-            fbg = getattr(frame,'_frameBGs',None)
             s = c.getSpaceAfter()
             if getattr(c,'_SPACETRANSFER',False):
                 s = pS
             pS = s
-            if fbg:
-                fbgl, fbgr, fbgc = fbg[-1]
+            fbg = getattr(frame,'_frameBGs',None)
+            if fbg and fbg[-1].active:
+                bg = fbg[-1]
+                fbgl = bg.left
+                fbgr = bg.right
+                bgm = bg.start
                 fbw = scale*(frame._width-fbgl-fbgr)
+                fbx = x0+scale*(fbgl-frame._leftPadding)
                 fbh = y + h + pS
                 fby = max(y0,y-pS)
                 fbh = max(0,fbh-fby)
-                if abs(fbw)>_FUZZ and abs(fbh)>_FUZZ:
-                    canv.saveState()
-                    canv.setFillColor(fbgc)
-                    canv.rect(x0+scale*(fbgl-frame._leftPadding)-0.1,fby-0.1,fbw+0.2,fbh+0.2,stroke=0,fill=1)
-                    canv.restoreState()
+                bg.render(canv,frame,fbx,fby,fbw,fbh)
             c._frame = frame
             c.drawOn(canv,x,y,_sW=aW-w)
             if c is not content[-1] and not getattr(c,'_SPACETRANSFER',None):
@@ -1216,6 +1220,11 @@ class _FindSplitterMixin:
         atTop = 1
         F = self._getContent(content)
         for i,f in enumerate(F):
+            if hasattr(f,'frameAction'):
+                from reportlab.platypus.doctemplate import Indenter
+                if isinstance(f,Indenter):
+                    availWidth -= f.left+f.right
+                continue
             w,h = f.wrapOn(canv,availWidth,0xfffffff)
             if w<=_FUZZ or h<=_FUZZ: continue
             W = max(W,w)
@@ -1380,12 +1389,93 @@ class ImageAndFlowables(_Container,_FindSplitterMixin,Flowable):
             aW, aH = self._wrapArgs
             _Container.drawOn(self, canv, x, y-self._aH,content=self._C1, aW=aW)
 
+class _AbsRect(NullDraw):
+    _ZEROSIZE=1
+    _SPACETRANSFER = True
+    def __init__(self,x,y,width,height,strokeWidth=0,strokeColor=None,fillColor=None,strokeDashArray=None):
+        self._x = x
+        self._y = y
+        self._width = width
+        self._height = height
+        self._strokeColor = strokeColor
+        self._fillColor = fillColor
+        self._strokeWidth = strokeWidth
+        self._strokeDashArray = strokeDashArray
+
+    def wrap(self, availWidth, availHeight):
+        return 0,0
+
+    def drawOn(self, canv, x, y, _sW=0):
+        if self._width>_FUZZ and self._height>_FUZZ:
+            st = self._strokeColor and self._strokeWidth is not None and self._strokeWidth>=0
+            if st or self._fillColor:
+                canv.saveState()
+                if st:
+                    canv.setStrokeColor(self._strokeColor)
+                    canv.setLineWidth(self._strokeWidth)
+                if self._fillColor:
+                    canv.setFillColor(self._fillColor)
+                canv.rect(self._x,self._y,self._width,self._height,stroke=1 if st else 0, fill=1 if self._fillColor else 0)
+                canv.restoreState()
+
+class _ExtendBG(NullDraw):
+    _ZEROSIZE=1
+    _SPACETRANSFER = True
+    def __init__(self,y,height,bg,frame):
+        self._y = y
+        self._height = height
+        self._bg = bg
+
+    def wrap(self, availWidth, availHeight):
+        return 0,0
+
+    def frameAction(self, frame):
+        bg = self._bg
+        fby = self._y
+        fbh = self._height
+        fbgl = bg.left
+        fbw = frame._width - fbgl - bg.right
+        fbx = frame._x1 - fbgl
+        canv = self.canv
+        pn = canv.getPageNumber()
+        bg.render(canv,frame,fbx,fby,fbw,fbh)
+
+class _AbsLine(NullDraw):
+    _ZEROSIZE=1
+    _SPACETRANSFER = True
+    def __init__(self,x,y,x1,y1,strokeWidth=0,strokeColor=None,strokeDashArray=None):
+        self._x = x
+        self._y = y
+        self._x1 = x1
+        self._y1 = y1
+        self._strokeColor = strokeColor
+        self._strokeWidth = strokeWidth
+        self._strokeDashArray = strokeDashArray
+
+    def wrap(self, availWidth, availHeight):
+        return 0,0
+
+    def drawOn(self, canv, x, y, _sW=0):
+        if self._strokeColor and self._strokeWidth is not None and self._strokeWidth>=0:
+            canv.saveState()
+            canv.setStrokeColor(self._strokeColor)
+            canv.setLineWidth(self._strokeWidth)
+            canv.line(self._x,self._y,self._x1,self._y1)
+            canv.restoreState()
+
 class BalancedColumns(_FindSplitterMixin,NullDraw):
     '''combine a list of flowables and an Image'''
     def __init__(self, F, nCols=2, needed=72, spaceBefore=0, spaceAfter=0, showBoundary=None,
             leftPadding=None, innerPadding=None, rightPadding=None, topPadding=None, bottomPadding=None,
-            name='', endSlack=0.1):
-        self.name = name
+            name='', endSlack=0.1,
+            boxStrokeColor=None,
+            boxStrokeWidth=0,
+            boxFillColor=None,
+            boxMargin=None,
+            vLinesStrokeColor=None,
+            vLinesStrokeWidth=None,
+            ):
+        self.name = name or 'BalancedColumns-%d' % id(self)
         if nCols <2:
             raise ValueError('nCols should be at least 2 not %r in %s' % (nCols,self.identitity()))
         self._content = _flowableSublist(F)
@@ -1400,6 +1490,12 @@ class BalancedColumns(_FindSplitterMixin,NullDraw):
         self._needed = needed - _FUZZ
         self.showBoundary = showBoundary
         self.endSlack = endSlack    #what we might allow as a lastcolumn overrun
+        self._boxStrokeColor = boxStrokeColor
+        self._boxStrokeWidth = boxStrokeWidth
+        self._boxFillColor = boxFillColor
+        self._boxMargin = boxMargin
+        self._vLinesStrokeColor = vLinesStrokeColor
+        self._vLinesStrokeWidth = vLinesStrokeWidth
 
     def identity(self, maxLen=None):
         return "<%s nCols=%r at %s%s%s>" % (self.__class__.__name__, self._nCols, hex(id(self)), self._frameName(),
@@ -1415,19 +1511,22 @@ class BalancedColumns(_FindSplitterMixin,NullDraw):
     def _generated_content(self,aW,aH):
         G = []
         frame = self._frame
-        from reportlab.platypus.doctemplate import CurrentFrameFlowable,LayoutError, ActionFlowable
+        from reportlab.platypus.doctemplate import CurrentFrameFlowable,LayoutError, ActionFlowable, Indenter
         from reportlab.platypus.frames import Frame
         from reportlab.platypus.doctemplate import FrameBreak
         lpad = frame._leftPadding if self._leftPadding is None else self._leftPadding
         rpad = frame._rightPadding if self._rightPadding is None else self._rightPadding
         tpad = frame._topPadding if self._topPadding is None else self._topPadding
         bpad = frame._bottomPadding if self._bottomPadding is None else self._bottomPadding
+        leftExtraIndent = frame._leftExtraIndent
+        rightExtraIndent = frame._rightExtraIndent
         gap = max(lpad,rpad) if self._innerPadding is None else self._innerPadding
         hgap = gap*0.5
         canv = self.canv
         nCols = self._nCols
-        cw = (aW - gap*(nCols-1))/float(nCols)
-        aH += frame._bottomPadding - bpad + frame._topPadding - tpad
+        cw = (aW - gap*(nCols-1) - lpad - rpad)/float(nCols)
+        aH0 = aH
+        aH -= tpad + bpad
         W,H0,_C0,C2 = self._findSplit(canv,cw,nCols*aH,paraFix=False)
         if not _C0:
             raise ValueError(
@@ -1451,7 +1550,7 @@ class BalancedColumns(_FindSplitterMixin,NullDraw):
                     if i==icheck:
                         wc, hc, cc0, cc1 = self._findSplit(canv,cw,2*ah,content=c1,paraFix=False)
                         if hc<=(1+endSlack)*ah:
-                            c.append([KeepInFrame(cw,ah,c1,mode='shrink')])
+                            c.append(c1)
                             h = ah-1e-6
                             cn = []
                             break
@@ -1522,13 +1621,19 @@ class BalancedColumns(_FindSplitterMixin,NullDraw):
         ftop = y1+bpad+tpad+aH
         fh = H1 + bpad + tpad
         y2 = ftop - fh
-        dx = fw / float(nCols)
+        dx = aW / float(nCols)
+        if leftExtraIndent or rightExtraIndent:
+            indenter0 = Indenter(-leftExtraIndent,-rightExtraIndent)
+            indenter1 = Indenter(leftExtraIndent,rightExtraIndent)
+        else:
+            indenter0 = indenter1 = None
 
         showBoundary=self.showBoundary if self.showBoundary is not None else frame.showBoundary
-        F = [Frame(x1+i*dx,y2,cw+(lpad+hgap if not i else rpad+hgap if i==nCols-1 else gap),fh,
+        obx = x1+leftExtraIndent+frame._leftPadding
+        F = [Frame(obx+i*dx,y2,dx,fh,
                 leftPadding=lpad if not i else hgap, bottomPadding=bpad,
                 rightPadding=rpad if i==nCols-1 else hgap, topPadding=tpad,
-                id=i,
+                id='%s-%d' %(self.name,i),
                 showBoundary=showBoundary,
                 overlapAttachedSpace=frame._oASpace,
                 _debug=frame._debug) for i in xrange(nCols)]
@@ -1538,20 +1643,88 @@ class BalancedColumns(_FindSplitterMixin,NullDraw):
         T=self._doctemplateAttr('pageTemplate')
         if T is None:
             raise LayoutError('%s used in non-doctemplate environment' % self.identity())
-        class Unwrapper(ActionFlowable):
-            def __init__(self):
+
+        BGs = getattr(frame,'_frameBGs',None)
+        xbg = bg = BGs[-1] if BGs else None
+
+        class TAction(ActionFlowable):
+            def __init__(self,bgs=[],F=[],f=None):
                 Flowable.__init__(self)
-            def apply(canv,doc,T=T,oldFrames=T.frames):
-                T.frames=oldFrames
-        T.frames=F
-        G.append(CurrentFrameFlowable(F[0].id))
+                self.bgs = bgs
+                self.F = F
+                self.f = f
+
+            def apply(self,doc,T=T):
+                T.frames = self.F
+                frame._frameBGs = self.bgs
+                doc.handle_currentFrame(self.f.id)
+                frame._frameBGs = self.bgs
+
+        if bg:
+            #G.append(Spacer(1e-5,1e-5))
+            #G[-1].__id__ = 'spacer0'
+            xbg = _ExtendBG(y2,fh,bg,frame)
+            G.append(xbg)
+
+        oldFrames = T.frames
+        G.append(TAction([],F,F[0]))
+        if indenter0: G.append(indenter0)
+        doBox = (self._boxStrokeColor and self._boxStrokeWidth and self._boxStrokeWidth>=0) or self._boxFillColor
+        doVLines = self._vLinesStrokeColor and self._vLinesStrokeWidth and self._vLinesStrokeWidth>=0
+        if doBox or doVLines:
+            obm = self._boxMargin
+            if not obm: obm = (0,0,0,0)
+            if len(obm)==1:
+                obmt = obml = obmr = obmb = obm[0]
+            elif len(obm)==2:
+                obmt = obmb = obm[0]
+                obml = obmr = obm[1]
+            elif len(obm)==3:
+                obmt = obm[0]
+                obml = obmr = obm[1]
+                obmb = obm[2]
+            elif len(obm)==4:
+                obmt = obm[0]
+                obmr = obm[1]
+                obmb = obm[2]
+                obml = obm[3]
+            else:
+                raise ValueError('Invalid value %s for boxMargin' % repr(obm))
+            obx1 = obx - obml
+            obx2 = F[-1]._x1+F[-1]._width + obmr
+            oby2 = y2-obmb
+            obh = fh+obmt+obmb
+            oby1 = oby2+obh
+            if doBox:
+                box = _AbsRect(obx1,oby2, obx2-obx1, obh,
+                        fillColor=self._boxFillColor,
+                        strokeColor=self._boxStrokeColor,
+                        strokeWidth=self._boxStrokeWidth,
+                        )
+            if doVLines:
+                vLines = []
+                for i in xrange(1,nCols):
+                    vlx = 0.5*(F[i]._x1 + F[i-1]._x1+F[i-1]._width)
+                    vLines.append(_AbsLine(vlx,oby2,vlx,oby1,strokeWidth=self._vLinesStrokeWidth,strokeColor=self._vLinesStrokeColor))
+        else:
+            oby1 = ftop
+            oby2 = y2
+
+        if doBox: G.append(box)
+        if doVLines: G.extend(vLines)
+        sa = self.getSpaceAfter()
         for i in xrange(nCols):
-            G.append(KeepInFrame(W1,H1,C[i],mode='shrink'))
+            Ci = KeepInFrame(W1,H1,C[i],mode='shrink')
+            sa = max(sa,Ci.getSpaceAfter())
+            G.append(Ci)
             if i!=nCols-1:
                 G.append(FrameBreak)
-            else:
-                G.append(Unwrapper())
-                G.append(CurrentFrameFlowable(frame.id))
+        G.append(TAction(BGs,oldFrames,frame))
+        if xbg:
+            if C1: sa = 0
+            xbg._y = min(y2,oby2) - sa
+            xbg._height = max(ftop,oby1) - xbg._y
+        if indenter1: G.append(indenter1)
         if C1:
             G.append(
                 BalancedColumns(C1, nCols=nCols,
@@ -1562,9 +1735,16 @@ class BalancedColumns(_FindSplitterMixin,NullDraw):
                     rightPadding=self._rightPadding,
                     topPadding=self._topPadding,
                     bottomPadding=self._bottomPadding,
+                    name=self.name+'-1', endSlack=self.endSlack,
+                    boxStrokeColor=self._boxStrokeColor,
+                    boxStrokeWidth=self._boxStrokeWidth,
+                    boxFillColor=self._boxFillColor,
+                    boxMargin=self._boxMargin,
+                    vLinesStrokeColor=self._vLinesStrokeColor,
+                    vLinesStrokeWidth=self._vLinesStrokeWidth,
                     )
                 )
-        return H1, G
+        return fh, G
 
     def wrap(self,aW,aH):
         #here's where we mess with everything
@@ -1595,12 +1775,54 @@ class AnchorFlowable(Spacer):
     def draw(self):
         self.canv.bookmarkHorizontal(self._name,0,0)
 
+class _FBGBag(ABag):
+    def matches(self,frame,canv):
+        fid = id(frame)
+        return ((isinstance(self.fid,list) and fid in self.fid or fid==self.fid)
+                    and id(canv)==self.cid and self.pn==canv.getPageNumber())
+
+    def getDims(self,canv):
+        self._inst = canv._code[self.codePos].split()
+        return map(float,self._inst[1:5])
+
+    def setDims(self,canv,x,y,w,h):
+        self._inst[1:5] = [fp_str(x,y,w,h)]
+        canv._code[self.codePos] = ' '.join(self._inst)
+
+    def render(self,canv,frame,fbx,fby,fbw,fbh):
+        if abs(fbw)>_FUZZ and abs(fbh)>_FUZZ:
+            pn = canv.getPageNumber()
+            if self.fid==id(frame) and self.cid==id(canv) and self.pn==pn:
+                ox,oy,ow,oh = self.getDims(canv)
+                self.setDims(canv,ox,fby,ow,oh+oy-fby)
+            else:
+                canv.saveState()
+                fbgc = self.fillColor
+                if fbgc:
+                    canv.setFillColor(fbgc)
+                sw = self.strokeWidth
+                sc = None if sw is None or sw<0 else self.strokeColor
+                if sc:
+                    canv.setStrokeColor(sc)
+                    canv.setLineWidth(sw)
+                    da = self.strokeDashArray
+                    if da:
+                        canv.setDash(da)
+                self.fid = id(frame)
+                self.cid = id(canv)
+                self.pn = pn
+                self.codePos = len(canv._code)
+                canv.rect(fbx,fby,fbw,fbh,stroke=1 if sc else 0,fill=1 if fbgc else 0)
+                canv.restoreState()
+
 class FrameBG(AnchorFlowable):
     """Start or stop coloring the frame background
     left & right are distances from the edge of the frame to start stop colouring.
+    if start in ('frame','frame-permanent') then the background is filled from here to the bottom of the frame and immediately discarded
+    for the frame case.
     """
     _ZEROSIZE=1
-    def __init__(self, color=None, left=0, right=0, start=True):
+    def __init__(self, color=None, left=0, right=0, start=True, strokeWidth=None, strokeColor=None, strokeDashArray=None):
         Spacer.__init__(self,0,0)
         self.start = start
         if start:
@@ -1608,6 +1830,9 @@ class FrameBG(AnchorFlowable):
             self.left = _evalMeasurement(left)
             self.right = _evalMeasurement(right)
             self.color = color
+            self.strokeWidth = strokeWidth
+            self.strokeColor = strokeColor
+            self.strokeDashArray = strokeDashArray
 
     def __repr__(self):
         return "%s(%s)" % (self.__class__.__name__,', '.join(['%s=%r' % (i,getattr(self,i,None)) for i in 'start color left right'.split()]))
@@ -1616,8 +1841,23 @@ class FrameBG(AnchorFlowable):
         frame = getattr(self,'_frame',None)
         if frame is None: return
         if self.start:
-            w = getattr(frame,'_lineWidth',0)
-            frame._frameBGs.append((self.left,self.right,self.color))
+            sc = self.strokeColor
+            sw = self.strokeWidth
+            sw = -1 if sw is None else sw
+            frame._frameBGs.append(
+                        _FBGBag(left=self.left,
+                                right=self.right,
+                                fillColor=self.color,
+                                start=self.start if self.start in ('frame','frame-permanent') else None,
+                                strokeColor=self.strokeColor,
+                                strokeWidth=self.strokeWidth,
+                                strokeDashArray=self.strokeDashArray,
+                                fid = 0,
+                                cid = 0,
+                                pn = -1,
+                                codePos = None,
+                                active=True,
+                                ))
         elif frame._frameBGs:
             frame._frameBGs.pop()
 
@@ -1628,11 +1868,12 @@ class FrameSplitter(NullDraw):
     that are listed in nextFrames and switch to the first of those frames.
     '''
     _ZEROSIZE=1
-    def __init__(self,nextTemplate,nextFrames=[],gap=10,required=72):
-        self.nextTemplate=nextTemplate
-        self.nextFrames=nextFrames or []
-        self.gap=gap
-        self.required=required
+    def __init__(self, nextTemplate, nextFrames=[], gap=10, required=72, adjustHeight=True):
+        self.nextTemplate = nextTemplate
+        self.nextFrames = nextFrames or []
+        self.gap = gap
+        self.required = required
+        self.adjustHeight = adjustHeight
 
     def wrap(self,aW,aH):
         frame = self._frame
@@ -1664,7 +1905,7 @@ class FrameSplitter(NullDraw):
             h=aH-self.gap
             for i,f in enumerate(F):
                 f=copy(f)
-                f.height=h
+                if self.adjustHeight: f.height=h
                 f._reset()
                 F[i]=f
             T.frames=F
@@ -1936,9 +2177,14 @@ class ListFlowable(_Container,Flowable):
                     start = self._bulletType
                     auto = True
         if self._bulletType!='bullet':
-            for v in start:
-                if v not in self._numberStyles:
-                    raise ValueError('invalid start %r' % start)
+            if auto:
+                for v in start:
+                    if v not in self._numberStyles:
+                        raise ValueError('invalid start=%r or bullettype=%r' % (start,self._bulletType))
+            else:
+                for v in self._bulletType:
+                    if v not in self._numberStyles:
+                        raise ValueError('invalid bullettype=%r' % self._bulletType)
         self._start = start
         self._auto = auto or isinstance(start,(list,tuple))
 
